package com.cloudbility.openapi.client;

import com.cloudbility.openapi.CloudApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.request.CreateCloudRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;

import java.util.List;
import java.util.Map;

public class DefaultCloudApi extends AbstractApi implements CloudApi {

    public DefaultCloudApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public CloudVo getCloudById(Long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/" + cloudId)
                .build(), CloudVo.class);
    }

    @Override
    public CloudVo getCloudDetail(Long cloudId, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/detail")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramIfNotNull("scope", scope)
                .build(), CloudVo.class);
    }

    @Override
    public GetRegionResult getRegionByProvider(CloudProvider provider) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/region/byProvider/" + provider.name())
                .build(), GetRegionResult.class);
    }

    @Override
    public GetRegionResult getRegionByCloud(Long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/region/byCloud/" + cloudId)
                .build(), GetRegionResult.class);
    }

    @Override
    public GetProjectResult getProjectsByCloud(Long cloudId, String regionId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/projects")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .build(), GetProjectResult.class);
    }

    @Override
    public InstanceTypeResult listAvailableInstanceTypes(Long cloudId, String regionId, String zoneId, CloudInstanceChargeType instanceChargeType) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/availableInstanceTypes")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("zoneId", zoneId)
                .paramShouldNotEmpty("instanceChargeType", instanceChargeType)
                .build(), InstanceTypeResult.class);
    }

    @Override
    public DiskTypeResult listAvailableDiskTypes(Long cloudId, String regionId, String zoneId, String instanceType) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/availableDiskTypes")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("zoneId", zoneId)
                .paramIfNotNull("instanceType", instanceType)
                .build(), DiskTypeResult.class);
    }

    @Override
    public CreateCloudResult createClouds(List<CreateCloudVo> cloudVos) throws OpenapiException {
        if (cloudVos == null || cloudVos.size() == 0) {
            throw new IllegalArgumentException("cloudVos is empty");
        }
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/cloud").body(new CreateCloudRequest(cloudVos))
                .build(), CreateCloudResult.class);
    }

    @Override
    public CreateCloudResult createCloud(CreateCloudVo cloudVo) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/cloudV2")
                .paramShouldNotEmpty("teamId", cloudVo.getTeamId())
                .paramShouldNotEmpty("name", cloudVo.getName())
                .paramIfNotNull("creatorId", cloudVo.getCreatorId())
                .paramIfNotNull("roles", cloudVo.getRoles())
                .paramIfNotNull("cidrRule", cloudVo.getCidrRule())
                .paramIfNotNull("cidrs", cloudVo.getCidrs())
                .paramIfNotNull("referencedPolicyId", cloudVo.getReferencedPolicyId())
                .build(), CreateCloudResult.class);
    }

    @Override
    public ListCloudsResult getCloudByTeam(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/byTeam/" + teamId)
                .build(), ListCloudsResult.class);
    }

    @Override
    public ListCloudsResult getCloudByTeamCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/cloud/byTeamCode/" + teamCode)
                .build(), ListCloudsResult.class);
    }

    @Override
    public CloudVo createAzure(long teamId, String name, Long creatorId, String roles, String subscriptionId, String tenantId, String clientId, String clientSecret) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/cloud/azure")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("name", name)

                .paramIfNotNull("creatorId", creatorId)
                .paramIfNotNull("roles", roles)

                .paramShouldNotEmpty("subscriptionId", subscriptionId)
                .paramShouldNotEmpty("tenantId", tenantId)
                .paramShouldNotEmpty("clientId", clientId)
                .paramShouldNotEmpty("clientSecret", clientSecret)
                .build(), CloudVo.class);

    }

    @Override
    public CloudVo createOpenstack(long teamId, String name, Long creatorId, String roles, String authUrl, String username, String password, String domainName, String projectId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/cloud/openstack")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("creatorId", creatorId)
                .paramIfNotNull("roles", roles)

                .paramShouldNotEmpty("authUrl", authUrl)
                .paramShouldNotEmpty("username", username)
                .paramShouldNotEmpty("password", password)
                .paramShouldNotEmpty("domainName", domainName)
                .paramShouldNotEmpty("projectId", projectId)

                .build(), CloudVo.class);
    }

    @Override
    public CloudVo createPublic(long teamId, String name, Long creatorId, String roles, String accessKeyId, String accessKeySecret, CloudProvider provider) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/cloud/public")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("creatorId", creatorId)
                .paramIfNotNull("roles", roles)

                .paramShouldNotEmpty("akId", accessKeyId)
                .paramShouldNotEmpty("akSecret", accessKeySecret)
                .paramShouldNotEmpty("provider", provider)
                .build(), CloudVo.class);
    }

    @Override
    public void updateNameOrRoles(long cloudId, String name, String roles) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/cloud/" + cloudId)
                .paramIfNotNull("name", name)
                .paramIfNotNull("roles", roles)
                .build(), Void.class);
    }

    @Override
    public void updateRoles(long cloudId, String roles) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/cloud/" + cloudId + "/roles")
                .paramIfNotNull("roles", roles)
                .build(), Void.class);
    }

    @Override
    public void updateDepartments(long cloudId, String departments) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/cloud/" + cloudId + "/departments")
                .paramIfNotNull("departments", departments)
                .build(), Void.class);
    }

    @Override
    public void updateMembers(long cloudId, String members) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/cloud/" + cloudId + "/members")
                .paramIfNotNull("members", members)
                .build(), Void.class);
    }

    @Override
    public void deleteClouds(List<Long> cloudIds) throws OpenapiException {
        if (cloudIds == null || cloudIds.size() == 0) {
            throw new IllegalArgumentException("teamIds is empty");
        }
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/cloud/" + StringUtil.join(cloudIds, ","))
                .build(), Void.class);
    }

    @Override
    public void revokeAllRolesFromCloud(long cloudId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/cloud/" + cloudId + "/revokeAllRoles")
                .build(), Void.class);
    }


    @Override
    public GetProxyScriptResult getProxyScript(long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/cloud/" + cloudId + "/proxyScript")
                .build(), GetProxyScriptResult.class);
    }

    @Override
    public void updateIntranetAccessType(long cloudId, CloudIntranetAccessType cloudIntranetAccessType) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/cloud/" + cloudId + "/intranetAccessType")
                .paramShouldNotEmpty("intranetAccessType", cloudIntranetAccessType)
                .build(), Void.class);
    }

    @Override
    public Long updateLanCloudCidrRule(long cloudId, CloudCidrRule cidrRule, String cidrs) throws OpenapiException {
        JobVo jobVo = execute(RequestorBuilder.create()
                .put("/cloud/" + cloudId + "/cidrRule")
                .paramShouldNotEmpty("cidrRule", cidrRule)
                .paramIfNotNull("cidrs", cidrs)
                .build(), JobVo.class);
        return jobVo.getId();
    }

}
