package com.cloudbility.openapi.client;

import com.cloudbility.openapi.*;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.request.ImportUserRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;

public class DefaultUserApi extends AbstractApi implements UserApi {

    public DefaultUserApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public ImportUserResult importUsers(List<ImportUserVo> users) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/user")
                .body(new ImportUserRequest(users))
                .build(), ImportUserResult.class);
    }

    @Override
    public ImportUserResult importUser(ImportUserVo userVo) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/userV2")
                .paramShouldNotEmpty("name", userVo.getName())
                .paramIfNotNull("introduce", userVo.getIntroduce())
                .paramIfNotNull("gender", userVo.getGender())
                .paramIfNotNull("address", userVo.getAddress())
                .paramIfNotNull("password", userVo.getPassword())
                .paramIfNotNull("forceUpdate", userVo.isForceUpdate())
                .paramIfNotNull("joinTeam", userVo.getJoinTeam())
                .paramIfNotNull("roles", userVo.getRoles())
                .paramIfNotNull("expirationDate", userVo.getExpirationDate())
                .build(), ImportUserResult.class);
    }

    @Override
    public UserBriefResult getUserBriefList(Long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/findIds").paramIfNotNull("teamId", teamId)
                .build(), UserBriefResult.class);
    }


    @Override
    public PagedUserListResult getUsersByTeam(Long teamId, int page, int size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/byTeam")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), PagedUserListResult.class);
    }

    @Override
    public UserListResult getUserByNickname(String nickname) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/byNickname")
                .paramShouldNotEmpty("nickname", nickname)
                .build(), UserListResult.class);
    }

    @Override
    public UserBriefResult getUserBriefListByTeamCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/ids/byTeamCode/" + teamCode)
                .build(), UserBriefResult.class);
    }


    @Override
    public GetUserResult getUser(long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/" + userId)
                .build(), GetUserResult.class);
    }

    @Override
    public PagedUserListResult getGetUserList(Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/users").paramIfNotNull("page", page)
                .paramIfNotNull("size", size)
                .build(), PagedUserListResult.class);
    }

    @Override
    public GetFunctionResult getUserFunctionsByTeam(Long teamId, Long userId) throws OpenapiException {
        return this.getUserFunctionsByTeam(teamId, userId, null, null);
    }

    @Override
    public GetFunctionResult getUserFunctionsByJwtAndTeam(Long teamId, String jwt) throws OpenapiException {
        return this.getUserFunctionsByJwtAndTeam(teamId, jwt, null, null);
    }


    @Override
    public GetFunctionResult getUserFunctionsByTeam(Long teamId, Long userId, String module, FunctionType type) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/functions/byUser")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("userId", userId)
                .paramIfNotNull("module", module)
                .paramIfNotNull("type", type)
                .build(), GetFunctionResult.class);
    }

    @Override
    public GetFunctionResult getUserFunctionsByJwtAndTeam(Long teamId, String jwt, String module, FunctionType type) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/functions/byJwt")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("jwt", jwt)
                .paramIfNotNull("module", module)
                .paramIfNotNull("type", type)
                .build(), GetFunctionResult.class);
    }

    @Override
    public TeamListResult getJoinedTeams(long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/" + userId + "/joinedTeams")
                .build(), TeamListResult.class);
    }

    @Override
    public RoleListResult findJoinedRoles(Long teamId, Long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/user/" + userId + "/joinedRoles")
                .build(), RoleListResult.class);
    }

    @Override
    public RoleListResult findJoinedRoles(Long teamId, String account) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/user/byAccount/" + account + "/joinedRoles")
                .build(), RoleListResult.class);
    }

    @Override
    public UserListResult getUsersByName(String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/users/byName/" + name)
                .paramShouldNotEmpty("name", name)
                .build(), UserListResult.class);
    }

    @Override
    public UserListResult getUsersByNameLike(String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/users/byNameLike")
                .paramShouldNotEmpty("name", name)
                .build(), UserListResult.class);
    }

    @Override
    public UserListResult getUsersByCreateTime(Date createTimeFrom, Date createTimeTo) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/users/byCreateTime")
                .paramShouldNotEmpty("fromTime", DateUtil.formatDate(createTimeFrom))
                .paramShouldNotEmpty("toTime", DateUtil.formatDate(createTimeTo))
                .build(), UserListResult.class);
    }

    @Override
    public UserListResult getUsersByCursor(int cursor, int size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/users/byCursor")
                .paramShouldNotEmpty("cursorId", cursor)
                .paramShouldNotEmpty("size", size)
                .build(), UserListResult.class);
    }

    @Override
    public GetUserResult getUserByAccount(String account) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/user/byAccount/" + account)
                .build(), GetUserResult.class);
    }

    @Override
    public GetUserResult getUserByPhone(String phone) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/byPhone/" + phone)
                .build(), GetUserResult.class);
    }

    @Override
    public GetUserResult getUserByEmail(String email) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/byEmail/" + email)
                .build(), GetUserResult.class);
    }

    @Override
    public void updateUserByAccount(String account, UpdateUserVo updateUserVo) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/byAccount/" + account)
                .body(updateUserVo)
                .build(), Void.class);
    }


    @Override
    public void updateUserEmailByAccount(String account, String email) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/byAccount/" + account + "/email")
                .paramIfNotNull("email", email)
                .build(), Void.class);
    }

    @Override
    public void updateUserPasswordByAccount(String account, String password) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/byAccount/" + account + "/password")
                .paramIfNotNull("password", password)
                .build(), Void.class);
    }

    @Override
    public void updateUserEmail(long userId, String email) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/" + userId + "/email")
                .paramIfNotNull("email", email)
                .build(), Void.class);
    }

    @Override
    public void updateUserPassword(long userId, String password) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/" + userId + "/password")
                .paramIfNotNull("password", password)
                .build(), Void.class);
    }

    @Override
    public void updateUserAccount(long userId, String newAccount) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/" + userId + "/account")
                .paramIfNotNull("account", newAccount)
                .build(), Void.class);
    }

    @Override
    public void updateUserAccount(String account, String newAccount) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/user/byAccount/" + account + "/account")
                .paramIfNotNull("newAccount", newAccount)
                .build(), Void.class);
    }

    @Override
    public void blockUser(long userId, int blockingSeconds, String description) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/user/" + userId + "/blocked")
                .paramShouldNotEmpty("blockingSeconds", blockingSeconds)
                .paramIfNotNull("description", description)
                .build(), Void.class);

    }

    @Override
    public void unblockUser(long userId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/user/" + userId + "/unblocked")
                .build(), Void.class);
    }

    @Override
    public void blockUser(String account, int blockingSeconds, String description) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/user/byAccount/" + account + "/blocked")
                .paramShouldNotEmpty("blockingSeconds", blockingSeconds)
                .paramIfNotNull("description", description)
                .build(), Void.class);
    }

    @Override
    public void unblockUser(String account) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/user/byAccount/" + account + "/unblocked")
                .build(), Void.class);
    }

    @Override
    public void deleteUserByAccount(String account) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/user/byAccount/" + account)
                .build(), Void.class);
    }

    @Override
    public void deleteUsers(List<Long> userIds) throws OpenapiException {
        if (userIds == null || userIds.size() == 0) {
            throw new IllegalArgumentException("userIds is empty");
        }
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/user/" + StringUtil.join(userIds, ","))
                .build(), Void.class);
    }

    @Override
    public void verifyIdCard(String realName, String idCard) throws OpenapiException {
        try {
            realName = URLEncoder.encode(realName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/verifyIdCard/" + realName + "/" + idCard)
                .build(), Void.class);
    }

    @Override
    public int countOnlineUsers() throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                .get("/users/countOnline")
                .build(), CountResult.class);
        return result.getCount();
    }

    @Override
    public int countOnlineMembers(long teamId) throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                .get("/users/countOnlineMembers")
                .paramShouldNotEmpty("teamId", teamId)
                .build(), CountResult.class);
        return result.getCount();
    }

    @Override
    public boolean isUserInTeam(Long teamId, Long userId) throws OpenapiException {
        BooleanResponse response =  execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/isUserInTeam")
                .paramIfNotNull("teamId", teamId)
                .paramIfNotNull("userId", userId)
                .build(), BooleanResponse.class);
        return response.getValue();
    }

    @Override
    public WorkTaskVo.UserExtendParamVo saveOrUpdateUserExtendParam(Long userId, String paramName, String paramValue, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/user/saveOrUpdateUserExtendParam")
                .paramShouldNotEmpty("userId", userId)
                .paramShouldNotEmpty("paramName", paramName)
                .paramShouldNotEmpty("paramValue", paramValue)
                .paramIfNotNull("description", description)
                .build(), WorkTaskVo.UserExtendParamVo.class);
    }

    @Override
    public int deleteUserExtendParam(Long userId, String paramName) throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/user/deleteUserExtendParam")
                .paramShouldNotEmpty("userId", userId)
                .paramIfNotNull("paramName", paramName)
                .build(), CountResult.class);
        return result.getCount();
    }

    @Override
    public WorkTaskVo.UserExtendParamResult getUserExtendParams(Long userId, String paramName, String paramValue) throws OpenapiException {
        if(userId == null && StringUtil.isEmpty(paramName) && StringUtil.isEmpty(paramValue)){
            throw new OpenapiException(OpenapiErrorCode.InvalidParameter("三个参数不能全部为空"));
        }
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/getUserExtendParams")
                .paramIfNotNull("userId", userId)
                .paramIfNotNull("paramName", paramName)
                .paramIfNotNull("paramValue", paramValue)
                .build(), WorkTaskVo.UserExtendParamResult.class);
    }

    @Override
    public UserListResult getUserListByExtendParam(String paramName, String paramValue) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/getUserListByExtendParam")
                .paramShouldNotEmpty("paramName", paramName)
                .paramShouldNotEmpty("paramValue", paramValue)
                .build(), UserListResult.class);
    }

    @Override
    public SimpleUserVo createUser(String account, String nickname, String phone, String email, String password, String thirdPartyUserId, String joinTeamCode, String roleIds, Long departmentId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/user/create")
                .paramShouldNotEmpty("account", account)
                .paramIfNotNull("nickname", nickname)
                .paramIfNotNull("phone", phone)
                .paramIfNotNull("email", email)
                .paramIfNotNull("password", password)
                .paramIfNotNull("thirdPartyUserId", thirdPartyUserId)
                .paramIfNotNull("joinTeamCode", joinTeamCode)
                .paramIfNotNull("roleIds", roleIds)
                .paramIfNotNull("departmentId", departmentId)
                .build(), SimpleUserVo.class);
    }

    @Override
    public SimpleUserVo queryByThirdPartyUserId(String thirdPartyUserId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/user/queryByThirdPartyUserId")
                .paramShouldNotEmpty("thirdPartyUserId", thirdPartyUserId)
                .build(), SimpleUserVo.class);
    }
}
