package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.VolumeApi;
import com.cloudbility.openapi.enums.zstack.DeleteModeType;
import com.cloudbility.openapi.enums.zstack.UsingStateEventType;
import com.cloudbility.openapi.enums.zstack.VolumeProvisioningStrategyType;
import com.cloudbility.openapi.enums.zstack.VolumeStatusType;
import com.cloudbility.openapi.response.DataDiskListResult;
import com.cloudbility.openapi.response.DiskOfferingListResult;
import com.cloudbility.openapi.vo.DataDiskVo;
import com.cloudbility.openapi.vo.Scope;
import org.apache.commons.collections.CollectionUtils;
import org.zstack.sdk.DiskOfferingInventory;

import java.util.Set;

public class DefaultVolumeApi extends AbstractApi implements VolumeApi {
    public DefaultVolumeApi(OpenapiClient client) {
        super(client);
    }


    @Override
    public DataDiskVo createZStackVolume(String clientToken,
                                         Long cloudId,
                                         Scope scope,
                                         String name,
                                         String diskOfferingId,
                                         String description,
                                         String primaryStorageId,
                                         VolumeProvisioningStrategyType provisioningStrategy) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/volume/cloud/" + cloudId + "/zstackCreate")
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("diskOfferingId", diskOfferingId)
                .paramIfNotNull("description", description)
                .paramIfNotNull("primaryStorageId", primaryStorageId)
                .paramIfNotNull("provisioningStrategy", provisioningStrategy)
                .paramIfNotNull("clientToken", clientToken)
                .build(), DataDiskVo.class);
    }

    @Override
    public DataDiskListResult listZStackVolumes(Long cloudId,
                                                Scope scope,
                                                String instanceId,
                                                Set<String> diskIdSet,
                                                String name,
                                                VolumeStatusType volumeStatus) throws OpenapiException {
        String diskIdsString = CollectionUtils.isEmpty(diskIdSet) ? null : String.join(",", diskIdSet);
        return execute(RequestorBuilder.create()
                .get("/volume/cloud/" + cloudId + "/zstackList")
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("diskIdSet", diskIdsString)
                .paramIfNotNull("instanceId", instanceId)
                .paramIfNotNull("name", name)
                .paramIfNotNull("volumeStatus", volumeStatus)
                .build(), DataDiskListResult.class);
    }


    @Override
    public DiskOfferingListResult listZStackDiskOfferings(Long cloudId,
                                                          Scope scope,
                                                          Set<String> diskOfferingIds,
                                                          String name,
                                                          Long diskSizeInMb) throws OpenapiException {
        String diskOfferingIdsString = CollectionUtils.isEmpty(diskOfferingIds) ? null : String.join(",", diskOfferingIds);
        return execute(RequestorBuilder.create()
                .get("/volume/cloud/" + cloudId + "/zstackDiskOfferings")
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("diskOfferingIds", diskOfferingIdsString)
                .paramIfNotNull("name", name)
                .paramIfNotNull("diskSizeInMb", diskSizeInMb)
                .build(), DiskOfferingListResult.class);
    }

    @Override
    public DiskOfferingListResult changeZStackDiskOfferings(String clientToken,
                                                            Long cloudId,
                                                            Scope scope,
                                                            Set<String> diskOfferingIds,
                                                            UsingStateEventType stateEvent) throws OpenapiException {
        String diskOfferingIdsString = CollectionUtils.isEmpty(diskOfferingIds) ? null : String.join(",", diskOfferingIds);
        return execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/volume/cloud/" + cloudId + "/zstackDiskOfferingsState")
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("diskOfferingIds", diskOfferingIdsString)
                .paramShouldNotEmpty("stateEvent", stateEvent)
                .paramIfNotNull("clientToken", clientToken)
                .build(), DiskOfferingListResult.class);
    }

    @Override
    public void deleteZStackDiskOfferings(String clientToken,
                                          Long cloudId,
                                          Scope scope,
                                          Set<String> diskOfferingIds,
                                          DeleteModeType deleteMode) throws OpenapiException {
        String diskOfferingIdsString = CollectionUtils.isEmpty(diskOfferingIds) ? null : String.join(",", diskOfferingIds);
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/volume/cloud/" + cloudId + "/zstackDiskOfferings")
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("diskOfferingIds", diskOfferingIdsString)
                .paramShouldNotEmpty("deleteMode", deleteMode)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public DiskOfferingInventory createZStackDiskOffering(String clientToken, Long cloudId,
                                                          Scope scope,
                                                          String name,
                                                          Long diskSizeInMb,
                                                          String description,
                                                          Long readBandwidthInMb,
                                                          Long writeBandwidthInMb,
                                                          Long totalBandwidthInMb) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/volume/cloud/" + cloudId + "/zstackDiskOffering")
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("diskSizeInMb", diskSizeInMb)
                .paramIfNotNull("description", description)
                .paramIfNotNull("readBandwidthInMb", readBandwidthInMb)
                .paramIfNotNull("writeBandwidthInMb", writeBandwidthInMb)
                .paramIfNotNull("totalBandwidthInMb", totalBandwidthInMb)
                .paramIfNotNull("clientToken", clientToken)
                .build(), DiskOfferingInventory.class);
    }
}
