package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.BooleanVo;
import com.cloudbility.openapi.vo.CreateCredentialRequest;
import com.cloudbility.openapi.vo.CredentialHostBrief;
import com.cloudbility.openapi.vo.CredentialVo;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;

@Api(tags = "30 Credential api", description = "登录凭证API")
public interface CredentialApi extends Openapi {

    @ApiOperation(value = "创建一个登录凭证", response = CredentialVo.class, notes = "参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示将登录凭证授权给id=12的角色和团队管理员角色。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/credential", method = RequestMethod.POST)
    @ApiResponses(value = {
            @ApiResponse(code = 201, message = "Credential created successfully"),
            @ApiResponse(code = 400, message = "parameter error"),
            @ApiResponse(code = 409, message = "credential name conflict"),
    })
    CredentialVo createCredential(@RequestBody CreateCredentialRequest request) throws OpenapiException;

    @ApiOperation(value = "修改一个登录凭证", response = CredentialVo.class, notes = "参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示将创建的登录凭证授权给id=12的角色和团队管理员角色。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/credential/{credentialId}", method = RequestMethod.PUT)
    CredentialVo updateCreateCredential(@ApiParam(value = "登录凭证ID", required = true) @PathVariable("credentialId") Long credentialId,
                                        @ApiParam(value = "凭证名称,团队唯一。不传入时不更新") @RequestParam(value = "name", required = false) String name,
                                        @ApiParam(value = "登录账号,不传入时不更新") @RequestParam(value = "account", required = false) String account,
                                        @ApiParam(value = "授权角色，不传入则不更新。注意，这个参数是替换授权而不是追加授权。") @RequestParam(value = "roles", required = false) String roles,
                                        @ApiParam(value = "操作者ID，不传入时，默认团队创建者") @RequestParam(value = "operator", required = false) Long operator,
                                        @ApiParam(value = "切换用户名，不传入时不更新") @RequestParam(value = "suUser", required = false) String suUser,
                                        @ApiParam(value = "切换用户使用的命令，不传入则不更新。") @RequestParam(value = "suCommand", required = false) String suCommand
    ) throws OpenapiException;


    @ApiOperation(value = "删除登录凭证")
    @RequestMapping(value = "/credential/{credentialId}", method = RequestMethod.DELETE)
    void deleteCredential(@ApiParam(value = "登录凭证ID", required = true) @PathVariable("credentialId") Long credentialId) throws OpenapiException;


    @ApiOperation(value = "获取登录凭证授权角色列表")
    @RequestMapping(value = "/credential/{credentialId}/roles", method = RequestMethod.GET)
    CredentialRolesResult getCredentialRoles(@ApiParam(value = "登录凭证ID", required = true) @PathVariable("credentialId") Long credentialId) throws OpenapiException;

    @ApiOperation(value = "获取团队登录凭证列表（根据团队code）", response = GetCredentialListResult.class)
    @RequestMapping(value = "/credential/byTeamCode/{teamCode:.+}", method = RequestMethod.GET)
    GetCredentialListResult getCredentialByTeam(@ApiParam(value = "团队code", required = true) @PathVariable("teamCode") String teamCode,
                                                @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "获取团队登录凭证列表（根据团队teamId）", response = GetCredentialListResult.class)
    @RequestMapping(value = "/credential/byTeam/{teamId}", method = RequestMethod.GET)
    GetCredentialListResult getCredentialByTeam(@ApiParam(value = "团队ID", required = true) @PathVariable("teamId") Long teamId,
                                                @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "获取主机登录凭证列表")
    @GetMapping(value = "/credential/byHost/{hostId}")
    GetCredentialListResult getCredentialByHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "获取团队主机凭证列表", response = GetHostCredentialListResult.class)
    @RequestMapping(value = "/credential/findTeamHostCrednetials/{teamId}", method = RequestMethod.GET)
    GetHostCredentialListResult getHostCredentialList(@ApiParam(value = "团队ID", required = true) @PathVariable("teamId") Long teamId,
                                                      @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                      @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size,
                                                      @ApiParam(value = "主机实例id") @RequestParam(value = "instanceId", defaultValue = "") String instanceId,
                                                      @ApiParam(value = "公网ip") @RequestParam(value = "publicIp", defaultValue = "") String publicIp,
                                                      @ApiParam(value = "内网ip") @RequestParam(value = "innerIp", defaultValue = "") String innerIp) throws OpenapiException;

    @ApiOperation(value = "修改主机凭证密码")
    @RequestMapping(value = "/credential/modifyCredentialPass", method = RequestMethod.POST)
    BooleanVo modifyHostCredentialPass(@ApiParam("主机凭证ID列表，英文逗号分隔") @RequestParam("credentialIds") String credentialIds,
                                       @ApiParam("凭证密码") @RequestParam("password") String password) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "获取登录凭证主机列表", response = CredentialHostsResult.class, notes = "v6.3后结构变更,该接口废弃")
    @RequestMapping(value = "/credential/{credentialId}/hosts", method = RequestMethod.GET)
    CredentialHostsResult getCredentialHosts(@ApiParam(value = "登录凭证ID", required = true) @PathVariable("credentialId") Long credentialId) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "添加主机到登录凭证（如果已经添加，则修改）", response = CredentialHostBrief.class, notes = "v6.3后结构变更,该接口废弃")
    @RequestMapping(value = "/credential/{credentialId}/{hostId}", method = RequestMethod.POST)
    CredentialHostBrief addCredentialHost(@ApiParam(value = "登录凭证ID", required = true) @PathVariable("credentialId") Long credentialId,
                                          @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                          @ApiParam(value = "访问端口，不传入时，使用协议默认端口") @RequestParam(value = "port", required = false) Integer port,
                                          @ApiParam(value = "登录密码，当认证类型是密码时，可以传入") @RequestParam(value = "password", required = false) String password,
                                          @ApiParam(value = "登录密钥，当认证类型是密钥时，可以传入") @RequestParam(value = "key", required = false) String key,
                                          @ApiParam(value = "登录密钥使用密钥对，当认证类型是密钥时，可以传入，不要与password、key重复传入") @RequestParam(value = "sshKeyId", required = false) Long sshKeyId,
                                          @ApiParam(value = "操作者ID") @RequestParam(value = "operator", required = false) Long operator,
                                          @ApiParam(value = "切换用户的密码，可选") @RequestParam(value = "suPassword", required = false) String suPassword
    ) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "删除登录凭证中的主机", response = DeleteCredentialHostResult.class, notes = "v6.3后结构变更,该接口废弃")
    @RequestMapping(value = "/credential/{credentialId}/{hostId}", method = RequestMethod.DELETE)
    DeleteCredentialHostResult deleteCredentialHost(@ApiParam(value = "登录凭证ID", required = true) @PathVariable("credentialId") Long credentialId,
                                                    @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId
    ) throws OpenapiException;

}
