package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.RoleDetailResult;
import com.cloudbility.openapi.vo.RoleVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "6 role api", description = "角色管理API")
public interface RoleApi extends Openapi {


    @ApiOperation(value = "获取团队所有角色", notes = "可以用 GET /roles代替")
    @RequestMapping(value = "/role", method = RequestMethod.GET)
    PagedRoleListResult getRoles(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "分页获取团队角色列表")
    @RequestMapping(value = "/roles", method = RequestMethod.GET)
    PagedRoleListResult getRolesByPage(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                       @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                       @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "获取团队所有角色（根据团队code）")
    @RequestMapping(value = "/role/byTeamCode/{teamCode:.+}", method = RequestMethod.GET)
    PagedRoleListResult getRolesByTeamCode(@ApiParam(value = "所属团队code", required = true) @PathVariable("teamCode") String teamCode) throws OpenapiException;

    @ApiOperation(value = "分页获取团队角色列表（根据团队code）")
    @RequestMapping(value = "/roles/byTeamCode/{teamCode:.+}", method = RequestMethod.GET)
    PagedRoleListResult getRolesByPage(@ApiParam(value = "所属团队code", required = true) @PathVariable("teamCode") String teamCode,
                                       @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                       @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;


    @ApiOperation(value = "获取用户在团队中加入的角色列表")
    @RequestMapping(value = "/role/byUser/{userId}", method = RequestMethod.GET)
    RoleListResult getRolesByUser(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                  @ApiParam(value = "用户ID", required = true) @PathVariable("userId") Long userId) throws OpenapiException;

    @ApiOperation(value = "获取用户在团队中加入的角色列表")
    @RequestMapping(value = "/role/byTeamCodeAndUserAccount", method = RequestMethod.GET)
    RoleListResult getByTeamCodeAndUserAccount(@ApiParam(value = "所属团队code", required = true) @RequestParam("teamCode") String teamCode,
                                               @ApiParam(value = "用户账号", required = true) @RequestParam("userAccount") String userAccount) throws OpenapiException;


    @ApiOperation(value = "获取团队中系统角色", notes = "只有系统内置角色才有code,而且同一个code的角色有且只有一个，目前系统内置三个角色：TEAM_CREATOR(团队创建者）、TEAM_ADMINER（管理员）、TEAM_MEMBER（所有成员）。私有部署版可以通过role_templates.json定义系统角色。")
    @RequestMapping(value = "/role/byCode/{code:.+}", method = RequestMethod.GET)
    RoleVo getRoleByCode(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                         @ApiParam(value = "角色Code", required = true) @PathVariable("code") String code) throws OpenapiException;

    @ApiOperation(value = "获取团队中系统角色（根据团队code）", notes = "只有系统内置角色才有code,而且同一个code的角色有且只有一个，目前系统内置三个角色：TEAM_CREATOR(团队创建者）、TEAM_ADMINER（管理员）、TEAM_MEMBER（所有成员）。私有部署版可以通过role_templates.json定义系统角色。")
    @RequestMapping(value = "/role/byRoleAndTeamCode", method = RequestMethod.GET)
    RoleVo getRoleByCode(@ApiParam(value = "所属团队code", required = true) @RequestParam("teamCode") String teamCode,
                         @ApiParam(value = "角色Code", required = true) @RequestParam("roleCode") String roleCode) throws OpenapiException;


    @ApiOperation(value = "创建角色")
    @RequestMapping(value = "/role", method = RequestMethod.POST)
    CreateRoleResult addRole(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                             @ApiParam(value = "角色名称", required = true) @RequestParam("name") String name,
                             @ApiParam(value = "角色描述") @RequestParam(value = "description", required = false) String description,
                             @ApiParam(value = "权限code列表，可选，用逗号连接") @RequestParam(value = "functionCodes", required = false) String functionCodes
    ) throws OpenapiException;

    @ApiOperation(value = "删除角色")
    @RequestMapping(value = "/role/{roleId}", method = RequestMethod.DELETE)
    void deleteRole(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId) throws OpenapiException;

    @ApiOperation(value = "获取角色详情")
    @RequestMapping(value = "/role/{roleId}", method = RequestMethod.GET)
    RoleDetailResult getRoleDetails(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "是否获取成员列表") @RequestParam(value = "includeMembers", required = false) boolean includeMembers,
            @ApiParam(value = "是否获取权限代码列表") @RequestParam(value = "includeFunctionCodes", required = false) boolean includeFunctionCodes) throws OpenapiException;

    @ApiOperation(value = "获取授权角色的用户列表", response = UserBriefResult.class)
    @RequestMapping(value = "/role/{roleId}/members", method = RequestMethod.GET)
    UserBriefResult getRoleMembers(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "获取角色可见的资源列表", notes = "目前只包含主机、cdn、bucket。后续版本就添加通用接口")
    @RequestMapping(value = "/role/{roleId}/resources", method = RequestMethod.GET)
    GetRoleResourcesResult getRoleResources(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId) throws OpenapiException;

    @ApiOperation(value = "授权角色给虚拟分组")
    @RequestMapping(value = "/role/{roleId}/vg/{vgId}", method = RequestMethod.POST)
    void grantRoleToVg(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "虚拟分组ID", required = true) @PathVariable("vgId") long vgId) throws OpenapiException;

    @ApiOperation(value = "授权角色给物理网络分组")
    @RequestMapping(value = "/role/{roleId}/hg/{hgId}", method = RequestMethod.POST)
    void grantRoleToHg(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "物理网络分组ID", required = true) @PathVariable("hgId") long hgId) throws OpenapiException;

    @ApiOperation(value = "授权角色给云账户")
    @RequestMapping(value = "/role/{roleId}/cloud/{cloudId}", method = RequestMethod.POST)
    void grantRoleToCloud(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") long cloudId) throws OpenapiException;

    @ApiOperation(value = "撤销角色给虚拟分组的授权", notes = "如果不存在授权，不会出错")
    @RequestMapping(value = "/role/{roleId}/vg/{vgId}", method = RequestMethod.DELETE)
    void revokeRoleFromVg(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "虚拟分组ID", required = true) @PathVariable("vgId") long vgId) throws OpenapiException;

    @ApiOperation(value = "撤销角色给物理网络分组的授权", notes = "如果不存在授权，不会出错")
    @RequestMapping(value = "/role/{roleId}/hg/{hgId}", method = RequestMethod.DELETE)
    void revokeRoleFromHg(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "物理网络分组ID", required = true) @PathVariable("hgId") long hgId) throws OpenapiException;

    @ApiOperation(value = "撤销角色给物理网络分组的授权", notes = "如果不存在授权，不会出错")
    @RequestMapping(value = "/role/{roleId}/cloud/{cloudId}", method = RequestMethod.DELETE)
    void revokeRoleFromCloud(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") long cloudId) throws OpenapiException;

    @ApiOperation(value = "授权角色给团队成员")
    @RequestMapping(value = "/role/{roleId}/members/{userIds}", method = RequestMethod.POST)
    void grantRoleToMember(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "用户ID，用','连接", required = true) @PathVariable("userIds") String userIdsStr) throws OpenapiException;

    @ApiOperation(value = "授权角色给一个团队成员")
    @RequestMapping(value = "/role/{roleId}/members/byAccount/{account:.+}", method = RequestMethod.POST)
    void grantRoleToMemberByAccount(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account) throws OpenapiException;

    @ApiOperation(value = "撤销用户的角色")
    @RequestMapping(value = "/role/{roleId}/members/{userIds}", method = RequestMethod.DELETE)
    void revokeRoleFromMembers(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "用户ID列表，用','连接", required = true) @PathVariable("userIds") String userIdOrAccounts) throws OpenapiException;

    @ApiOperation(value = "撤销用户的角色")
    @RequestMapping(value = "/role/{roleId}/members/byAccount/{account:.+}", method = RequestMethod.DELETE)
    void revokeRoleFromMembersByAccount(
            @ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account) throws OpenapiException;

    @ApiOperation(value = "获取系统功能权限列表")
    @RequestMapping(value = "/function", method = RequestMethod.GET)
    GetFunctionResult getFunctions();

    @ApiOperation(value = "获取角色的权限列表")
    @RequestMapping(value = "/role/{roleId}/functions", method = RequestMethod.GET)
    GetFunctionResult getFunctionsOfRole(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId) throws OpenapiException;

    @ApiOperation(value = "角色添加权限")
    @RequestMapping(value = "/role/{roleId}/functions/{functionCodes}", method = RequestMethod.POST)
    void addFunctionsToRole(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
                                       @ApiParam(value = "功能权限代码列表，用','连接", required = true) @PathVariable("functionCodes") String functionCodes) throws OpenapiException;

    @ApiOperation(value = "角色添加权限V2")
    @RequestMapping(value = "/role/{roleId}/functionsV2", method = RequestMethod.POST)
    void addFunctionsToRoleV2(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
                                         @ApiParam(value = "功能权限代码列表，用','连接", required = true) @RequestParam("functionCodes") String functionCodes) throws OpenapiException;

    @ApiOperation(value = "角色删除权限")
    @RequestMapping(value = "/role/{roleId}/functions/{functionCodes}", method = RequestMethod.DELETE)
    void deleteFunctionsFromRole(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
                                            @ApiParam(value = "功能权限代码列表，用','连接", required = true) @PathVariable("functionCodes") String functionCodes) throws OpenapiException;

    @ApiOperation(value = "角色删除权限V2")
    @RequestMapping(value = "/role/{roleId}/functionsV2", method = RequestMethod.DELETE)
    void deleteFunctionsFromRoleV2(@ApiParam(value = "角色ID", required = true) @PathVariable("roleId") long roleId,
                                              @ApiParam(value = "功能权限代码列表，用','连接", required = true) @RequestParam("functionCodes") String functionCodes) throws OpenapiException;


    @ApiOperation(value = "获取虚拟分组的角色列表")
    @RequestMapping(value = "/roles/byVg/{vgId}", method = RequestMethod.GET)
    PagedRoleListResult getRolesByVg(@ApiParam(value = "虚拟分组ID", required = true) @PathVariable("vgId") long vgId) throws OpenapiException;

    @ApiOperation(value = "获取物理分组的角色列表")
    @RequestMapping(value = "/roles/byHg/{hgId}", method = RequestMethod.GET)
    PagedRoleListResult getRolesByHg(@ApiParam(value = "物理分组ID", required = true) @PathVariable("hgId") long hgId) throws OpenapiException;

    @ApiOperation(value = "获取云账户的角色列表")
    @RequestMapping(value = "/roles/byCloud/{cloudId}", method = RequestMethod.GET)
    PagedRoleListResult getRolesByCloud(@ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") long cloudId) throws OpenapiException;

    @ApiOperation(value = "获取当前系统自定义的系统角色模版", response = GetRoleTemplatesResult.class, notes = "系统自定义角色模版在文件/opt/cloudbility/conf/role_templates.json中")
    @RequestMapping(value = "/roleTemplates", method = RequestMethod.GET)
    GetRoleTemplatesResult getRoleTemplates() throws OpenapiException;


    @ApiOperation(value = "更新角色的名称和描述")
    @RequestMapping(value = "/role/{id}", method = RequestMethod.PUT)
    void updateRole(@ApiParam(value = "角色ID", required = true) @PathVariable("id") long id,
                    @ApiParam(value = "角色名称", required = true) @RequestParam("name") String name,
                    @ApiParam(value = "角色描述") @RequestParam(value = "description", required = false) String description
    ) throws OpenapiException;


    @ApiOperation(value = "根据名称查询团队内的角色", notes = "因为角色名称唯一，所以最多返回一条记录")
    @RequestMapping(value = "/roles/byName", method = RequestMethod.GET)
    RoleListResult findRoleByName(@ApiParam(value = "角色名称", required = true) @RequestParam("name") String name,
                                  @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "根据名称模糊查询团队内的角色")
    @RequestMapping(value = "/roles/byNameLike", method = RequestMethod.GET)
    RoleListResult updateRole(@ApiParam(value = "角色名称包含的字符串", required = true) @RequestParam("name") String name,
                              @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

}
